@@ -488,19 +488,21 @@ static int stdout_stream_scan(
                 LineBreak force_flush,
                 size_t *ret_consumed) {
 
-        size_t consumed = 0, line_max;
+        size_t consumed = 0;
         int r;
 
         assert(s);
         assert(p);
 
-        line_max = stdout_stream_line_max(s);
 
         for (;;) {
                 LineBreak line_break;
                 size_t skip, found;
                 char *end1, *end2;
-                size_t tmp_remaining = MIN(remaining, line_max);
+                size_t tmp_remaining, line_max;
+
+                line_max = stdout_stream_line_max(s);
+                tmp_remaining = MIN(remaining, line_max);
 
                 end1 = memchr(p, '\n', tmp_remaining);
                 end2 = memchr(p, 0, end1 ? (size_t) (end1 - p) : tmp_remaining);
