@@ -313,7 +313,7 @@ int allocate_scope(
         if (r < 0)
                 return bus_log_parse_error(r);
 
-        r = bus_wait_for_jobs_one(w, object, false);
+        r = bus_wait_for_jobs_one(w, object, false, NULL);
         if (r < 0)
                 return r;
 
