@@ -72,7 +72,7 @@ EOF
 
     # Forward journal messages to the console, so we have something to investigate even if we fail to mount
     # the encrypted /var
-    mkdir "$initdir/etc/systemd/journald.conf.d/"
+    mkdir -p "$initdir/etc/systemd/journald.conf.d/"
     echo -ne "[Journal]\nForwardToConsole=yes\n" >"$initdir/etc/systemd/journald.conf.d/99-forward.conf"
 
     # If $INITRD wasn't provided explicitly, generate a custom one with dm-crypt
